/**
 * Defines macros for bitmask manipulation.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#include "tme_std.h"

#ifndef TME_FLAGS_H
#define TME_FLAGS_H

/**
 * Bitmask manipulation API.
 *
 * @note This is not a real data structure, though it is declared as such
 *       for better grouping in the generated documentation.
 *
 * @struct TmeFlags
 */

/**
 * Bitmask type.
 *
 * @param N Number of bits. Allowed values: 8, 16, 32 and 64.
 *
 * @public @related TmeFlags
 */
#define TmeFlags(N) uint##N##_t

/**
 * Empty bitmask.
 *
 * @public @related TmeFlags
 */
#define tme_flags_empty (0U)

/**
 * Returns a bitmask with the specified bit set.
 *
 * @param N Bit to set.
 * @return Bitmask with the specified bit set.
 *
 * @public @related TmeFlags
 */
#define tme_flags_bit(N) (1U << (unsigned)N)

/**
 * Use this directive to check whether a bitmask has specific bits set.
 *
 * @param OPTIONS Bitmask.
 * @param OPTION Bits to check.
 *
 * @return True if the bits are set, false otherwise.
 *
 * @public @related TmeFlags
 */
#define tme_flags_is_set(OPTIONS, OPTION) (((OPTIONS) & (OPTION)) == (OPTION))

/**
 * Use this directive to check whether a bitmask has any of the specified bits set.
 *
 * @param OPTIONS Bitmask.
 * @param OPTION Bits to check.
 *
 * @return True if at least one of the specified bits is set, false otherwise.
 *
 * @public @related TmeFlags
 */
#define tme_flags_is_any_set(OPTIONS, OPTION) (((OPTIONS) & (OPTION)) != 0)

/**
 * Use this directive to set bits in a bitmask.
 *
 * @param OPTIONS Bitmask.
 * @param OPTION Bits to set.
 *
 * @public @related TmeFlags
 */
#define tme_flags_set(OPTIONS, OPTION) ((OPTIONS) |= (OPTION))

/**
 * Use this directive to unset bits in a bitmask.
 *
 * @param OPTIONS Bitmask.
 * @param OPTION Bits to unset.
 *
 * @public @related TmeFlags
 */
#define tme_flags_unset(OPTIONS, OPTION) ((OPTIONS) &= ~(OPTION))

/**
 * Use this directive to toggle in a bitmask.
 *
 * @param OPTIONS Bitmask.
 * @param OPTION Bits to toggle.
 *
 * @public @related TmeFlags
 */
#define tme_flags_toggle(OPTIONS, OPTION) ((OPTIONS) ^= (OPTION))

#endif // TME_FLAGS_H
