/**
 * Defines the iterator API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_ITERATOR_H
#define TME_ITERATOR_H

#include "tme_std.h"

TME_BEGIN_DECLS

/**
 * Iterator API.
 *
 * A TmeIterator is a wrapper around a function that gets called for every element
 * matched by a query submitted to the reasoner. By providing a generic context pointer,
 * you can plug any data structure (loggers, collections, etc.) which allows
 * for arbitrarily complex queries.
 *
 * The iterator function returns a `boolean` that can be used to control iteration:
 * by returning `true` iteration goes on to the next element,
 * while returning `false` causes it to stop. This is useful if, for example,
 * you want to find the first element matching certain criteria.
 *
 * @note This is not a real data structure, though it is declared as such
 *       for better grouping in the generated documentation.
 *
 * @struct TmeIterator
 */

/**
 * Defines a new iterator type.
 *
 * @param NAME Name of the iterator type.
 * @param ... Arguments of the iterator function, besides the context pointer.
 *
 * @note Under normal circumstances you don't need to use this.
 *       It's used in the TmeIterator header to generate
 *       the concrete iterator types used throughout the API.
 *
 * @private @related TmeIterator
 */
#define TME_ITERATOR_DEF(NAME, ...)                                                                 \
    /** NAME iterator. */                                                                           \
    /** @extends TmeIterator */                                                                     \
    typedef tme_struct(NAME##Iterator) {                                                            \
        /** The iterator context, can be anything. */                                               \
        void *ctx;                                                                                  \
        /** Pointer to a function called by the iterator for every element. */                      \
        bool (*for_each)(void *ctx, __VA_ARGS__);                                                   \
    } const Tme##NAME##Iterator

/**
 * Initializes an iterator.
 *
 * @param CTX Iterator context, can be anything.
 * @param FOR_EACH Pointer to a function called by the iterator for every element.
 * @return Initialized iterator.
 *
 * @public @related TmeIterator
 */
#define tme_iterator_init(CTX, FOR_EACH) \
    { .ctx = (void *)(CTX), .for_each = (FOR_EACH) }

TME_END_DECLS

#endif // TME_ITERATOR_H
