/**
 * Declares the knowledge base API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_KB_H
#define TME_KB_H

#include "tme_atomic.h"
#include "tme_abox_entry_type.h"
#include "tme_iterator.h"
#include "tme_vocabulary.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeAxiomProvider);
tme_struct_decl(TmeKB);
tme_struct_decl(TmeSemDesc);
/// @endcond

/// @name KB

/**
 * Models a knowledge base.
 *
 * @warning Unless otherwise stated, TmeSemDesc instances returned by the KB must not
 *          be mutated unless they have been copied beforehand (via e.g. tme_sem_desc_copy()),
 *          since they generally directly reference its internal data structures.
 *
 * @struct TmeKB
 */

/// @name Iterators

TME_ITERATOR_DEF(TBox, TmeAtomicConcept concept, TmeSemDesc *desc);
TME_ITERATOR_DEF(ABox, TmeAtomicIndividual individual, TmeSemDesc *desc, TmeABoxEntryType type);
TME_ITERATOR_DEF(KBConcept, TmeAtomicConcept);
TME_ITERATOR_DEF(KBIndividual, TmeAtomicIndividual individual, TmeABoxEntryType type);

/// @name Loading

/**
 * Loads all the individuals obtained from an axiom provider.
 *
 * @param kb KB instance.
 * @param provider Axiom provider.
 * @param type Type of the loaded individuals.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_load_individuals(TmeKB *kb, TmeAxiomProvider const *provider, TmeABoxEntryType type);

/**
 * Loads an individual into the ABox.
 *
 * @param kb KB instance.
 * @param individual Individual identifier.
 * @param desc Semantic description of the specified individual.
 * @param type Type of the loaded individual.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_load_individual(TmeKB *kb, TmeAtomicIndividual individual,
                            TmeSemDesc *desc, TmeABoxEntryType type);

/**
 * Unloads the specified individual from the ABox.
 *
 * @param kb KB instance.
 * @param individual Individual to unload.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_unload_individual(TmeKB *kb, TmeAtomicIndividual individual);

/**
 * Unloads all the individuals of a certain type.
 *
 * @param kb KB instance.
 * @param type Type of the individuals to unload.
 * @param [out] unloaded Unloaded individuals.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_unload_individuals(TmeKB *kb, TmeABoxEntryType type,
                               Vector(TmeAtomicIndividual) *unloaded);

/**
 * Sets the type of the specified individual.
 *
 * @param kb KB instance.
 * @param individual Individual whose type should be set.
 * @param type Type of the individual.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_tell_individual_type(TmeKB *kb, TmeAtomicIndividual individual, TmeABoxEntryType type);

/// @name Querying

/**
 * Returns the number of concepts in the TBox.
 *
 * @param kb KB instance.
 * @return Number of concepts in the TBox.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
tme_uint_t tme_kb_get_tbox_size(TmeKB *kb);

/**
 * Returns the number of individuals in the ABox.
 *
 * @param kb KB instance.
 * @return Number of individuals in the ABox.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
tme_uint_t tme_kb_get_abox_size(TmeKB *kb);

/**
 * Returns the number of individuals of the specified type.
 *
 * @param kb KB instance.
 * @param type Individual type.
 * @return Number of individuals of the specified type.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
tme_uint_t tme_kb_get_individuals_count(TmeKB *kb, TmeABoxEntryType type);

/**
 * Returns all the concepts in the TBox.
 *
 * @param kb KB instance.
 * @param [out] concepts TBox concepts.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_get_concepts(TmeKB *kb, Vector(TmeAtomicConcept) *concepts);

/**
 * Returns all the individuals in the ABox.
 *
 * @param kb KB instance.
 * @param [out] individuals ABox individuals.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_get_individuals(TmeKB *kb, Vector(TmeAtomicIndividual) *individuals);

/**
 * Returns all the individuals in the ABox, split in resources and requests.
 *
 * @param kb KB instance.
 * @param [out] resources Resource individuals.
 * @param [out] requests Request individuals.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_get_individuals_type(TmeKB *kb, Vector(TmeAtomicIndividual) *resources,
                                 Vector(TmeAtomicIndividual) *requests);

/**
 * Returns the TBox concepts and their normalized descriptions.
 *
 * @param kb KB instance.
 * @param [out] tbox TBox concepts and their normalized descriptions.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_get_tbox(TmeKB *kb, UHash(TmeEntityTable) *tbox);

/**
 * Returns the ABox individuals and their normalized descriptions.
 *
 * @param kb KB instance.
 * @param abox ABox individuals and their normalized descriptions.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_get_abox(TmeKB *kb, UHash(TmeEntityTable) *abox);

/**
 * Returns the ABox split in resources and requests.
 *
 * @param kb KB instance.
 * @param [out] resources Resource individuals and their normalized descriptions.
 * @param [out] requests Request individuals and their normalized descriptions.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_get_abox_type(TmeKB *kb, UHash(TmeEntityTable) *resources,
                          UHash(TmeEntityTable) *requests);

/**
 * Returns the vocabulary.
 *
 * @param kb KB instance.
 * @return Vocabulary instance.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
TmeVocabulary tme_kb_get_vocabulary(TmeKB *kb);

/**
 * Returns the unfolded and normalized semantic description of the specified concept.
 *
 * @param kb KB instance.
 * @param concept Concept whose description should be retrieved.
 * @return Normalized description.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
TmeSemDesc* tme_kb_normalized_concept(TmeKB *kb, TmeAtomicConcept concept);

/**
 * Returns the unfolded and normalized semantic description of the specified individual.
 *
 * @param kb KB instance.
 * @param individual Individual whose description should be retrieved.
 * @return Normalized description.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
TmeSemDesc* tme_kb_normalized_individual(TmeKB *kb, TmeAtomicIndividual individual);

/**
 * Performs the unfolding and CNF normalization of the specified description.
 *
 * @param kb KB instance.
 * @param desc Semantic description to normalize.
 * @return True if the description is satisfiable, false otherwise.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
bool tme_kb_sem_desc_normalize(TmeKB *kb, TmeSemDesc *desc);

/**
 * Folds the specified description.
 *
 * @param kb KB instance.
 * @param desc Semantic description to fold.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_sem_desc_fold(TmeKB *kb, TmeSemDesc *desc);

/// @name Iteration

/**
 * Iterates over the TBox.
 *
 * @param kb KB instance.
 * @param iter TBox iterator.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_iterate_tbox(TmeKB *kb, TmeTBoxIterator *iter);

/**
 * Iterates over the ABox.
 *
 * @param kb KB instance.
 * @param iter ABox iterator.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_iterate_abox(TmeKB *kb, TmeABoxIterator *iter);

/**
 * Iterates over all the individuals of the specified type.
 *
 * @param kb KB instance.
 * @param type Individual type.
 * @param iter ABox iterator.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_iterate_individuals_type(TmeKB *kb, TmeABoxEntryType type, TmeABoxIterator *iter);

/**
 * Iterates over all the concepts in the TBox, ignoring their descriptions.
 *
 * @param kb KB instance.
 * @param iter Concept iterator.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_iterate_concept_ptrs(TmeKB *kb, TmeKBConceptIterator *iter);

/**
 * Iterates over all the individuals in the ABox, ignoring their descriptions.
 *
 * @param kb KB instance.
 * @param iter Individual iterator.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_iterate_individual_ptrs(TmeKB *kb, TmeKBIndividualIterator *iter);

/**
 * Iterates over all the individuals of the specified type, ignoring their description.
 *
 * @param kb KB instance.
 * @param type Individual type.
 * @param iter Individual iterator.
 *
 * @public @memberof TmeKB
 */
TME_PUBLIC
void tme_kb_iterate_individual_ptrs_type(TmeKB *kb, TmeABoxEntryType type,
                                         TmeKBIndividualIterator *iter);

TME_END_DECLS

#endif // TME_KB_H
