/**
 * Declares the logging API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_LOGGER_H
#define TME_LOGGER_H

#include "tme_string.h"
#include "tme_string_utils.h"
#include "vector.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeKB);
tme_struct_decl(TmeLogger);
tme_struct_decl(TmeReasoner);
tme_struct_decl(TmeStringProvider);
/// @endcond

/**
 * Provides logging facilities.
 *
 * A logger instance can log either to stdout, to file or to a memory buffer.
 *
 * @note The logger won't deallocate the string provider instance that gets passed to it
 *       during its allocation: you must deallocate it yourself.
 *
 * @struct TmeLogger
 */

/// Vector of chars.
VECTOR_DECL_SPEC(char, TME_PUBLIC)

/// @name Lifecycle

/**
 * Allocates a new logger instance that logs to stdout.
 *
 * @param provider String provider.
 * @return Logger instance.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
TmeLogger* tme_logger_alloc_console(TmeStringProvider const *provider);

/**
 * Allocates a new logger instance that logs to file.
 *
 * @param provider String provider.
 * @param path Path of the log file.
 * @return Logger instance.
 *
 * @note The path is copied by the logger, so you are still responsible for
 *       deallocating the string passed to this allocator.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
TmeLogger* tme_logger_alloc_file(TmeStringProvider const *provider, char const *path);

/**
 * Allocates a new logger instance that logs to an internal memory buffer.
 *
 * @param provider String provider.
 * @return Logger instance.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
TmeLogger* tme_logger_alloc_memory(TmeStringProvider const *provider);

/**
 * Allocates a new logger that logs nowhere.
 *
 * @return Logger instance.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
TmeLogger* tme_logger_alloc_null(void);

/**
 * Deallocates the logger instance.
 *
 * @param logger Logger instance.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_free(TmeLogger *logger);

/// @name State

/**
 * Clears the logger, removing anything it has logged in the past.
 * The effect depends on the logger type:
 *
 * - Console/Null: no effect.
 *
 * - File: empties the file.
 *
 * - Memory: empties the memory buffer.
 *
 * @param logger Logger instance.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_clear(TmeLogger *logger);

/**
 * Returns the memory buffer of this logger.
 *
 * @param logger Logger instance.
 * @return Internal memory buffer.
 *
 * @note Must only be used on a memory logger.
 * @note The returned buffer is not copied, so you must not free it.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
Vector(char)* tme_logger_get_memory(TmeLogger *logger);

/**
 * Returns the file path of this logger.
 *
 * @param logger Logger instance.
 * @return File path.
 *
 * @note Must only be used on a file logger.
 * @note The returned path is not copied, so you must not free it.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
char const* tme_logger_get_path(TmeLogger *logger);

/// @name Logging

/**
 * Logs the specified formatted string.
 *
 * @param logger Logger instance.
 * @param format Format string.
 * @param ... Format arguments.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_logf(TmeLogger *logger, char const *format, ...);

/**
 * Logs the specified counted string.
 *
 * @param logger Logger instance.
 * @param string String to log.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log(TmeLogger *logger, TmeString string);

/**
 * Logs the specified counted string followed by a newline character.
 *
 * @param logger Logger instance.
 * @param string String to log.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_logln(TmeLogger *logger, TmeString string);

/**
 * Logs the specified counted string, freeing it once done.
 *
 * @param logger Logger instance.
 * @param string String to log.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_free(TmeLogger *logger, TmeString string);

/**
 * Logs the specified counted string followed by a newline character, freeing it when done.
 *
 * @param logger Logger instance.
 * @param string String to log.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_logln_free(TmeLogger *logger, TmeString string);

/**
 * Logs the IRI associated with the specified entity.
 *
 * @param logger Logger instance.
 * @param entity_ptr Entity identifier.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_entity_iri(TmeLogger *logger, TmeEntityPtr entity_ptr);

/**
 * Logs the readable representation of the specified entity.
 *
 * @param logger Logger instance.
 * @param entity_ptr Entity identifier.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_entity(TmeLogger *logger, TmeEntityPtr entity_ptr);

/**
 * Logs the readable representation of the specified negated entity.
 *
 * @param logger Logger instance.
 * @param entity_ptr Entity identifier.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_negated_entity(TmeLogger *logger, TmeEntityPtr entity_ptr);

/**
 * Logs the readable representation of the specified cardinality restriction.
 *
 * @param logger Logger instance.
 * @param role Cardinality restriction.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_card_role(TmeLogger *logger, TmeCardinalityRole role);

/**
 * Logs the readable representation of the specified minimum cardinality restriction.
 *
 * @param logger Logger instance.
 * @param role Cardinality restriction.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_gt_role(TmeLogger *logger, TmeCardinalityRole role);

/**
 * Logs the readable representation of the specified maximum cardinality restriction.
 *
 * @param logger Logger instance.
 * @param role Cardinality restriction.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_lt_role(TmeLogger *logger, TmeCardinalityRole role);

/**
 * Logs the readable representation of the specified value restriction.
 *
 * @param logger Logger instance.
 * @param role Value restriction.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_uv_role(TmeLogger *logger, TmeUniversalRole role);

/**
 * Logs the readable representation of the specified semantic description.
 *
 * @param logger Logger instance.
 * @param desc Semantic description.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_sem_desc(TmeLogger *logger, TmeSemDesc const *desc);

/**
 * Logs the readable representation of the specified abduction result.
 *
 * @param logger Logger instance.
 * @param abduction Abduction result.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_abduction(TmeLogger *logger, TmeAbduction abduction);

/**
 * Logs the readable representation of the specified bonus result.
 *
 * @param logger Logger instance.
 * @param bonus Bonus result.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_bonus(TmeLogger *logger, TmeBonus bonus);

/**
 * Logs the readable representation of the specified contraction result.
 *
 * @param logger Logger instance.
 * @param contraction Contraction result.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_contraction(TmeLogger *logger, TmeContraction contraction);

/**
 * Logs the readable representation of the specified covering result.
 *
 * @param logger Logger instance.
 * @param composition Covering result.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_composition(TmeLogger *logger, TmeComposition composition);

/**
 * Logs the TBox of the specified KB.
 *
 * @param logger Logger instance.
 * @param kb KB whose TBox should be logged.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_tbox(TmeLogger *logger, TmeKB *kb);

/**
 * Logs the ABox of the specified KB.
 *
 * @param logger Logger instance.
 * @param kb KB whose ABox should be logged.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_abox(TmeLogger *logger, TmeKB *kb);

/**
 * Logs the taxonomy inferred by the reasoner instance.
 *
 * @param logger Logger instance.
 * @param reasoner Reasoner whose taxonomy should be logged.
 *
 * @note The reasoner must have already computed the taxonomy
 *       (via e.g. tme_reasoner_classify()).
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_taxonomy(TmeLogger *logger, TmeReasoner const *reasoner);

/**
 * Logs a prompt string followed by a formatted time interval.
 * Useful for logging benchmark results.
 *
 * @param logger Logger instance.
 * @param prompt Prompt string.
 * @param nanos Time interval in nanoseconds.
 * @param unit Desired time unit.
 *
 * @public @memberof TmeLogger
 */
TME_PUBLIC
void tme_logger_log_nanos(TmeLogger *logger, char const *prompt, uint64_t nanos, TmeTimeUnit unit);

TME_END_DECLS

#endif // TME_LOGGER_H
