/**
 * Declares the available non-standard reasoning tasks.
 *
 * These functions are generally invoked by the TmeReasoner API,
 * though they are exposed publicly for additional flexibility.
 *
 * @note Unless otherwise stated, semantic descriptions must be unfolded
 *       and normalized prior to calling these functions.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_NON_STANDARD_H
#define TME_NON_STANDARD_H

#include "tme_atomic.h"
#include "tme_results.h"

TME_BEGIN_DECLS

/// @cond
tme_struct_decl(TmeKB);
tme_struct_decl(TmeReasoner);
tme_struct_decl(TmeSemDesc);
/// @endcond

/**
 * Checks the compatibility between the specified resource and request.
 *
 * @param resource Resource.
 * @param request Request.
 * @param bottom_concept Identifier of the bottom concept.
 * @return True if the compatibility relation holds, false otherwise.
 *
 * @see tme_reasoner_individual_is_compatible()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
bool tme_sem_desc_is_compatible(TmeSemDesc const *resource, TmeSemDesc const *request,
                                TmeAtomicConcept bottom_concept);

/**
 * Computes the Concept Abduction between the specified resource and request.
 *
 * @param resource Resource.
 * @param request Request.
 * @return Abduction result.
 *
 * @see tme_reasoner_individual_abduce()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeAbduction tme_sem_desc_abduce(TmeSemDesc const *resource, TmeSemDesc const *request);

/**
 * Computes the maximum penalty associated with the specified semantic description.
 *
 * @param desc Semantic description.
 * @return Maximum penalty.
 *
 * @see tme_reasoner_individual_max_penalty()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
double tme_sem_desc_abduction_max_penalty(TmeSemDesc const *desc);

/**
 * Computes the Concept Bonus between the specified resource and request.
 *
 * @param resource Resource.
 * @param request Request.
 * @param reasoner Reasoner instance.
 * @return Bonus result.
 *
 * @see tme_reasoner_individual_bonus()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeBonus tme_sem_desc_bonus(TmeSemDesc const *resource, TmeSemDesc const *request,
                            TmeReasoner *reasoner);

/**
 * Computes the Concept Contraction between the specified resource and request.
 *
 * @param resource Resource.
 * @param request Request.
 * @param reasoner Reasoner instance.
 * @return Contraction result.
 *
 * @see tme_reasoner_individual_contract()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeContraction tme_sem_desc_contract(TmeSemDesc const *resource, TmeSemDesc const *request,
                                     TmeReasoner *reasoner);

/**
 * Computes the Concept Covering for the specified request.
 *
 * @param request Request.
 * @param kb KB instance.
 * @return Covering result.
 *
 * @see tme_reasoner_individual_covering()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeComposition tme_sem_desc_cover(TmeSemDesc const *request, TmeKB *kb);

/**
 * Computes the Concept Difference between the specified semantic descriptions.
 *
 * @param lhs LHS of the difference.
 * @param rhs RHS of the difference.
 * @param reasoner Reasoner instance.
 * @return Result of the difference service.
 *
 * @see tme_reasoner_individual_difference()
 *
 * @public @memberof TmeReasoner
 */
TME_PUBLIC
TmeSemDesc* tme_sem_desc_difference(TmeSemDesc const *lhs, TmeSemDesc const *rhs,
                                    TmeReasoner *reasoner);

TME_END_DECLS

#endif // TME_NON_STANDARD_H
