/**
 * Defines data structures used as results of non-standard inferences.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_RESULTS_H
#define TME_RESULTS_H

#include "tme_compat.h"
#include "uhash.h"

TME_BEGIN_DECLS

/// @cond
tme_hash_decl(TmeEntityTable);
tme_struct_decl(TmeSemDesc);
/// @endcond

/**
 * Stores the result of the "Concept Abduction" inference service.
 *
 * "Hypothesis" represents the part of the request that is not present in the supplied
 * resource. The more must be hypothesized, the higher the associated "penalty".
 *
 * @see tme_reasoner_individual_abduce()
 */
typedef tme_struct(TmeAbduction) {

    /// Hypothesis of the abduction result.
    TmeSemDesc *hypothesis;

    /// Penalty associated with the abduction result.
    double penalty;

} TmeAbduction;

/**
 * Destroys the specified abduction result, releasing its memory.
 *
 * @param abduction Abduction instance.
 *
 * @public @related TmeAbduction
 */
#define tme_abduction_deinit(abduction) (tme_sem_desc_free((abduction).hypothesis))


/**
 * Stores the result of the "Concept Bonus" inference service.
 *
 * "Surplus" represents the part of the resource that was not requested.
 * The larger the surplus, the higher the associated "incentive".
 *
 * @see tme_reasoner_individual_bonus()
 */
typedef tme_struct(TmeBonus) {

    /// Surplus of the bonus result.
    TmeSemDesc *surplus;

    /// Incentive associated with the bonus result.
    double incentive;

} TmeBonus;

/**
 * Destroys the specified bonus result, releasing its memory.
 *
 * @param bonus Bonus instance.
 *
 * @public @related TmeBonus
 */
#define tme_bonus_deinit(bonus) (tme_sem_desc_free((bonus).surplus))

/**
 * Stores the result of the "Concept Contraction" inference service.
 *
 * "Keep" represents the part of the request that is compatible with the supplied resource,
 * while "give up" is the part that must be given up in order to make it compatible with it.
 * The more must be given up in the request, the higher the associated "penalty".
 *
 * @see tme_reasoner_individual_contract()
 */
typedef tme_struct(TmeContraction) {

    /// Keep part of the contraction result.
    TmeSemDesc *keep;

    /// Give up part of the contraction result.
    TmeSemDesc *give_up;

    /// Penalty associated with the contraction result.
    double penalty;

} TmeContraction;

/**
 * Destroys the specified contraction result, releasing its memory.
 *
 * @param contraction Contraction instance.
 *
 * @public @related TmeContraction
 */
#define tme_contraction_deinit(contraction) do {                                                    \
    tme_sem_desc_free((contraction).keep);                                                          \
    tme_sem_desc_free((contraction).give_up);                                                       \
} while(0)

/**
* Stores the result of the "Concept Covering" inference service.
*
* Multiple resources are collected in order to cover the request. The uncovered
* part of the request can be queried as a separate TmeSemDesc instance.
*
* @see tme_reasoner_individual_covering()
*/
typedef tme_struct(TmeComposition) {

    /// Resources that (partially) cover the request.
    UHash(TmeEntityTable) *cover_resources;

    /// Part of the request that has nor been covered.
    TmeSemDesc *uncovered_request;

} TmeComposition;

/**
 * Destroys the specified covering result, releasing its memory.
 *
 * @param composition Composition instance.
 *
 * @public @related TmeComposition
 */
#define tme_composition_deinit(composition) do {                                                    \
    uhash_free(TmeEntityTable, (composition).cover_resources);                                      \
    tme_sem_desc_free((composition).uncovered_request);                                             \
} while(0)

TME_END_DECLS

#endif // TME_RESULTS_H
