/**
 * Defines the semantic description data structure and related API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_SEM_DESC_H
#define TME_SEM_DESC_H

#include "tme_atomic.h"
#include "tme_cardinality_role.h"
#include "tme_flags.h"
#include "tme_universal_role.h"

TME_BEGIN_DECLS

/// @name TmeSemDesc

/// @cond
typedef TmeFlags(8) TmeSemDescFlags;
typedef union TmeSemDescStorage TmeSemDescStorage;
/// @endcond

/// Models ALN concept expressions as the conjunction of Ccn, Cgt, Clt, Cuv components.
typedef tme_struct(TmeSemDesc) {
    /// @cond
    TmeSemDescFlags flags;
    TmeSemDescStorage *storage;
    /// @endcond
} TmeSemDesc;

/**
 * Allocates a new semantic description.
 *
 * @return Semantic description instance.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
TmeSemDesc* tme_sem_desc_alloc(void);

/**
 * Copies an existing semantic description.
 *
 * @param desc Description to copy.
 * @return Copied description.
 *
 * @note The copy is deep (value restrictions are recursively copied).
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
TmeSemDesc* tme_sem_desc_copy(TmeSemDesc const *desc);

/**
 * Deallocates the specified semantic description.
 *
 * @param desc Semantic description to deallocate.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
void tme_sem_desc_free(TmeSemDesc *desc);

/**
 * Shrinks the specified semantic description instance, ensuring it takes up the least
 * amount of memory necessary.
 *
 * @param desc Semantic description instance.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
void tme_sem_desc_shrink(TmeSemDesc *desc);

/**
 * Returns the number of conjuncts present in the specified semantic description.
 *
 * @param desc Semantic description instance.
 * @return Number of conjuncts.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
tme_uint_t tme_sem_desc_count(TmeSemDesc const *desc);

/**
 * Checks whether the specified semantic description has no conjuncts.
 *
 * @param desc Semantic description instance.
 * @return True if the semantic description has no conjuncts, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_is_empty(TmeSemDesc const *desc);

/**
 * Returns the number of atomic concepts present in the semantic description.
 *
 * @param desc Semantic description instance.
 * @return Number of atomic concepts.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_atom_count(desc) vector_count(tme_sem_desc_get_atoms(desc))

/**
 * Returns the number of negated concepts present in the semantic description.
 *
 * @param desc Semantic description instance.
 * @return Number of negated concepts.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_neg_count(desc) vector_count(tme_sem_desc_get_negs(desc))

/**
 * Returns the number of minimum cardinality restrictions present in the semantic description.
 *
 * @param desc Semantic description instance.
 * @return Number of minimum cardinality restrictions.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_gt_count(desc) vector_count(tme_sem_desc_get_gts(desc))

/**
 * Returns the number of maximum cardinality restrictions present in the semantic description.
 *
 * @param desc Semantic description instance.
 * @return Number of maximum cardinality restrictions.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_lt_count(desc) vector_count(tme_sem_desc_get_lts(desc))

/**
 * Returns the number of value restrictions present in the semantic description.
 *
 * @param desc Semantic description instance.
 * @return Number of value restrictions.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_uv_count(desc) vector_count(tme_sem_desc_get_uvs(desc))

/**
 * Returns the atomic concepts present in the semantic description.
 *
 * @param desc Semantic description instance.
 * @return Atomic concepts.
 *
 * @warning The returned pointer becomes invalid once the description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
Vector(TmeAtomicConcept) const * tme_sem_desc_get_atoms(TmeSemDesc const *desc);

/**
 * Returns the negated concepts present in the semantic description.
 *
 * @param desc Semantic description instance.
 * @return Negated concepts.
 *
 * @warning The returned pointer becomes invalid once the description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
Vector(TmeAtomicConcept) const * tme_sem_desc_get_negs(TmeSemDesc const *desc);

/**
 * Returns the minimum cardinality restrictions present in the semantic description.
 *
 * @param desc Semantic description instance.
 * @return Minimum cardinality restrictions.
 *
 * @warning The returned pointer becomes invalid once the description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
Vector(TmeCardinalityRole) const * tme_sem_desc_get_gts(TmeSemDesc const *desc);

/**
 * Returns the maximum cardinality restrictions present in the semantic description.
 *
 * @param desc Semantic description instance.
 * @return Maximum cardinality restrictions.
 *
 * @warning The returned pointer becomes invalid once the description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
Vector(TmeCardinalityRole) const * tme_sem_desc_get_lts(TmeSemDesc const *desc);

/**
 * Returns the value restrictions present in the semantic description.
 *
 * @param desc Semantic description instance.
 * @return Value restrictions.
 *
 * @warning The returned pointer becomes invalid once the description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
Vector(TmeUniversalRole) const * tme_sem_desc_get_uvs(TmeSemDesc const *desc);

/**
 * Checks whether the specified semantic description is equivalent to the bottom concept.
 *
 * @param desc Semantic description instance.
 * @param bottom Entity identifier of the bottom concept.
 * @return  True if the semantic description is equivalent to the bottom concept, false otherwise.
 *
 * @note The semantic description must be normalized.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_is_bottom(TmeSemDesc const *desc, TmeAtomicConcept bottom);

/**
 * Equality function for semantic descriptions.
 *
 * @param desc LHS of the equality relation.
 * @param other RHS of the equality relation.
 * @return True if the equality relation holds, false otherwise.
 *
 * @note This function has no semantic meaning, as it also accounts for the order in which
 *       concepts and roles appear. It's mainly meant to be used in data structures
 *       that require their elements to be equatable.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_equals(TmeSemDesc const *desc, TmeSemDesc const *other);

/**
 * Checks whether the semantic description contains the specified atomic concept.
 *
 * @param desc Semantic description instance.
 * @param atom Atomic concept.
 * @return True if the atomic concept was found, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_has_atom(TmeSemDesc const *desc, TmeAtomicConcept atom);

/**
 * Checks whether the semantic description contains the specified negated concept.
 *
 * @param desc Semantic description instance.
 * @param neg Negated concept.
 * @return True if the negated concept was found, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_has_neg(TmeSemDesc const *desc, TmeAtomicConcept neg);

/**
 * Retrieves the minimum cardinality of the specified role.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @return Minimum cardinality.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
tme_uint_t tme_sem_desc_get_gt_card(TmeSemDesc const *desc, TmeAtomicRole role);

/**
 * Retrieves the maximum cardinality of the specified role.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @return Maximum cardinality.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
tme_uint_t tme_sem_desc_get_lt_card(TmeSemDesc const *desc, TmeAtomicRole role);

/**
 * Retrieves the filler of the value restriction involving the specified role.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @return Filler of the value restriction involving the role, or NULL.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
TmeSemDesc* tme_sem_desc_get_uv_filler(TmeSemDesc const *desc, TmeAtomicRole role);

/**
 * Performs the intersection between two semantic descriptions.
 *
 * @param desc Semantic description instance.
 * @param source Semantic description to intersect 'desc' with.
 * @return True if the description was mutated, false otherwise.
 *
 * @note Only the first semantic description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add(TmeSemDesc *desc, TmeSemDesc const *source);

/**
 * Intersects the atomic concepts of two semantic descriptions.
 *
 * @param desc Semantic description instance.
 * @param source Semantic description to intersect 'desc' with.
 * @return True if the description was mutated, false otherwise.
 *
 * @note Only the first semantic description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add_atoms(TmeSemDesc *desc, TmeSemDesc const *source);

/**
 * Intersects the negated concepts of two semantic descriptions.
 *
 * @param desc Semantic description instance.
 * @param source Semantic description to intersect 'desc' with.
 * @return True if the description was mutated, false otherwise.
 *
 * @note Only the first semantic description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add_negs(TmeSemDesc *desc, TmeSemDesc const *source);

/**
 * Intersects the minimum cardinality restrictions of two semantic descriptions.
 *
 * @param desc Semantic description instance.
 * @param source Semantic description to intersect 'desc' with.
 * @return True if the description was mutated, false otherwise.
 *
 * @note Only the first semantic description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add_gts(TmeSemDesc *desc, TmeSemDesc const *source);

/**
 * Intersects the maximum cardinality restrictions of two semantic descriptions.
 *
 * @param desc Semantic description instance.
 * @param source Semantic description to intersect 'desc' with.
 * @return True if the description was mutated, false otherwise.
 *
 * @note Only the first semantic description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add_lts(TmeSemDesc *desc, TmeSemDesc const *source);

/**
 * Intersects the value restrictions of two semantic descriptions.
 *
 * @param desc Semantic description instance.
 * @param source Semantic description to intersect 'desc' with.
 * @return True if the description was mutated, false otherwise.
 *
 * @note Only the first semantic description is mutated.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add_uvs(TmeSemDesc *desc, TmeSemDesc const *source);

/**
 * Intersects the specified semantic description with an atomic concept.
 *
 * @param desc Semantic description instance.
 * @param atom Atomic concept to intersect.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add_atom(TmeSemDesc *desc, TmeAtomicConcept atom);

/**
 * Intersects the specified semantic description with a negated concept.
 *
 * @param desc Semantic description instance.
 * @param neg Negated concept to intersect.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add_neg(TmeSemDesc *desc, TmeAtomicConcept neg);

/**
 * Intersects the specified semantic description with a minimum cardinality restriction.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @param cardinality Minimum cardinality.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add_gt(TmeSemDesc *desc, TmeAtomicRole role, tme_uint_t cardinality);

/**
 * Intersects the specified semantic description with a maximum cardinality restriction.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @param cardinality Maximum cardinality.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add_lt(TmeSemDesc *desc, TmeAtomicRole role, tme_uint_t cardinality);

/**
 * Intersects the specified semantic description with a value restriction.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @param filler Filler of the value restriction.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_add_uv(TmeSemDesc *desc, TmeAtomicRole role, TmeSemDesc const *filler);

/**
 * Sets the minimum cardinality of the specified role.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @param cardinality Minimum cardinality.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_set_gt(TmeSemDesc *desc, TmeAtomicRole role, tme_uint_t cardinality);

/**
 * Sets the maximum cardinality of the specified role.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @param cardinality Maximum cardinality.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_set_lt(TmeSemDesc *desc, TmeAtomicRole role, tme_uint_t cardinality);

/**
 * Sets the filler of the specified value restriction.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @param filler Filler of the value restriction.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_set_uv(TmeSemDesc *desc, TmeAtomicRole role, TmeSemDesc const *filler);

/**
 * Removes an atomic concept from the specified semantic description.
 *
 * @param desc Semantic description instance.
 * @param concept Atomic concept to remove.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_atom(TmeSemDesc *desc, TmeAtomicConcept concept);

/**
 * Removes a negated concept from the specified semantic description.
 *
 * @param desc Semantic description instance.
 * @param concept Negated concept to remove.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_neg(TmeSemDesc *desc, TmeAtomicConcept concept);

/**
 * Removes the minimum cardinality restriction of the specified role.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_gt(TmeSemDesc *desc, TmeAtomicRole role);

/**
 * Removes the maximum cardinality restriction of the specified role.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_lt(TmeSemDesc *desc, TmeAtomicRole role);

/**
 * Removes the value restriction of the specified role.
 *
 * @param desc Semantic description instance.
 * @param role Atomic role.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_uv(TmeSemDesc *desc, TmeAtomicRole role);

/**
 * Removes the atomic concept at the specified index.
 *
 * @param desc Semantic description instance.
 * @param idx Index of the atomic concept to remove.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
void tme_sem_desc_remove_atom_at(TmeSemDesc *desc, tme_uint_t idx);

/**
 * Removes the negated concept at the specified index.
 *
 * @param desc Semantic description instance.
 * @param idx Index of the negated concept to remove.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
void tme_sem_desc_remove_neg_at(TmeSemDesc *desc, tme_uint_t idx);

/**
 * Removes the minimum cardinality restriction at the specified index.
 *
 * @param desc Semantic description instance.
 * @param idx Index of the minimum cardinality restriction to remove.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
void tme_sem_desc_remove_gt_at(TmeSemDesc *desc, tme_uint_t idx);

/**
 * Removes the maximum cardinality restriction at the specified index.
 *
 * @param desc Semantic description instance.
 * @param idx Index of the maximum cardinality restriction to remove.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
void tme_sem_desc_remove_lt_at(TmeSemDesc *desc, tme_uint_t idx);

/**
 * Removes the value restriction at the specified index.
 *
 * @param desc Semantic description instance.
 * @param idx Index of the value restriction to remove.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
void tme_sem_desc_remove_uv_at(TmeSemDesc *desc, tme_uint_t idx);

/**
 * Removes all conjuncts from the semantic description.
 *
 * @param desc Semantic description instance.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_all(TmeSemDesc *desc);

/**
 * Removes all the atomic concepts.
 *
 * @param desc Semantic description instance.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_all_atoms(TmeSemDesc *desc);

/**
 * Removes all the negated concepts.
 *
 * @param desc Semantic description instance.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_all_negs(TmeSemDesc *desc);

/**
 * Removes all the minimum cardinality restrictions.
 *
 * @param desc Semantic description instance.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_all_gts(TmeSemDesc *desc);

/**
 * Removes all the maximum cardinality restrictions.
 *
 * @param desc Semantic description instance.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_all_lts(TmeSemDesc *desc);

/**
 * Removes all the value restrictions.
 *
 * @param desc Semantic description instance.
 * @return True if the description was mutated, false otherwise.
 *
 * @public @memberof TmeSemDesc
 */
TME_PUBLIC
bool tme_sem_desc_remove_all_uvs(TmeSemDesc *desc);

/**
 * Iterates over the atoms present in the semantic description, executing the specified code
 * block for each element.
 *
 * @param desc Semantic description instance.
 * @param concept Name of the variable holding the concept.
 * @param i Name of the index variable.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_iterate_atoms(desc, concept, i, code) \
    vector_iterate(TmeAtomicConcept, tme_sem_desc_get_atoms(desc), concept, i, code)

/**
 * Iterates over the atoms present in the semantic description in reverse order,
 * executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param concept Name of the variable holding the concept.
 * @param i Name of the index variable.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_iterate_atoms_reverse(desc, concept, i, code) \
    vector_iterate_reverse(TmeAtomicConcept, tme_sem_desc_get_atoms(desc), concept, i, code)

/**
 * Iterates over the atoms present in the semantic description, executing the specified code
 * block for each element.
 *
 * @param desc Semantic description instance.
 * @param concept Name of the variable holding the concept.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_foreach_atom(desc, concept, code) \
    vector_foreach(TmeAtomicConcept, tme_sem_desc_get_atoms(desc), concept, code)

/**
 * Iterates over the negated concepts present in the semantic description, executing
 * the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param neg Name of the variable holding the negated concept.
 * @param i Name of the index variable.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_iterate_negs(desc, neg, i, code) \
    vector_iterate(TmeAtomicConcept, tme_sem_desc_get_negs(desc), neg, i, code)

/**
 * Iterates over the negated concepts present in the semantic description in reverse order,
 * executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param neg Name of the variable holding the negated concept.
 * @param i Name of the index variable.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_iterate_negs_reverse(desc, neg, i, code) \
    vector_iterate_reverse(TmeAtomicConcept, tme_sem_desc_get_negs(desc), neg, i, code)

/**
 * Iterates over the negated concepts present in the semantic description, executing
 * the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param neg Name of the variable holding the negated concept.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_foreach_neg(desc, neg, code) \
    vector_foreach(TmeAtomicConcept, tme_sem_desc_get_negs(desc), neg, code)

/**
 * Iterates over the minimum cardinality restrictions present in the semantic description,
 * executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param gt_role Name of the variable holding the minimum cardinality restriction.
 * @param i Name of the index variable.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_iterate_gts(desc, gt_role, i, code) \
    vector_iterate(TmeCardinalityRole, tme_sem_desc_get_gts(desc), gt_role, i, code)

/**
 * Iterates over the minimum cardinality restrictions present in the semantic description
 * in reverse order, executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param gt_role Name of the variable holding the minimum cardinality restriction.
 * @param i Name of the index variable.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_iterate_gts_reverse(desc, gt_role, i, code) \
    vector_iterate_reverse(TmeCardinalityRole, tme_sem_desc_get_gts(desc), gt_role, i, code)

/**
 * Iterates over the minimum cardinality restrictions present in the semantic description,
 * executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param gt_role Name of the variable holding the minimum cardinality restriction.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_foreach_gt(desc, gt_role, code) \
    vector_foreach(TmeCardinalityRole, tme_sem_desc_get_gts(desc), gt_role, code)

/**
 * Iterates over the maximum cardinality restrictions present in the semantic description,
 * executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param lt_role Name of the variable holding the maximum cardinality restriction.
 * @param i Name of the index variable.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_iterate_lts(desc, lt_role, i, code) \
    vector_iterate(TmeCardinalityRole, tme_sem_desc_get_lts(desc), lt_role, i, code)

/**
 * Iterates over the maximum cardinality restrictions present in the semantic description
 * in reverse order, executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param lt_role Name of the variable holding the maximum cardinality restriction.
 * @param i Name of the index variable.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_iterate_lts_reverse(desc, lt_role, i, code) \
    vector_iterate_reverse(TmeCardinalityRole, tme_sem_desc_get_lts(desc), lt_role, i, code)

/**
 * Iterates over the maximum cardinality restrictions present in the semantic description,
 * executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param lt_role Name of the variable holding the maximum cardinality restriction.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_foreach_lt(desc, lt_role, code) \
    vector_foreach(TmeCardinalityRole, tme_sem_desc_get_lts(desc), lt_role, code)

/**
 * Iterates over the value restrictions present in the semantic description,
 * executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param uv_role Name of the variable holding the value restriction.
 * @param i Name of the index variable.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_iterate_uvs(desc, uv_role, i, code) \
    vector_iterate(TmeUniversalRole, tme_sem_desc_get_uvs(desc), uv_role, i, code)

/**
 * Iterates over the value restrictions present in the semantic description in reverse order,
 * executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param uv_role Name of the variable holding the value restriction.
 * @param i Name of the index variable.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_iterate_uvs_reverse(desc, uv_role, i, code) \
    vector_iterate_reverse(TmeUniversalRole, tme_sem_desc_get_uvs(desc), uv_role, i, code)

/**
 * Iterates over the value restrictions present in the semantic description,
 * executing the specified code block for each element.
 *
 * @param desc Semantic description instance.
 * @param uv_role Name of the variable holding the value restriction.
 * @param code Code block to execute.
 *
 * @warning You must not mutate the description while iterating.
 *
 * @public @related TmeSemDesc
 */
#define tme_sem_desc_foreach_uv(desc, uv_role, code) \
    vector_foreach(TmeUniversalRole, tme_sem_desc_get_uvs(desc), uv_role, code)

TME_END_DECLS

#endif // TME_SEM_DESC_H
