/**
 * Defines the counted string data structure and related API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_STRING_H
#define TME_STRING_H

#include "tme_std.h"
#include "tme_base_types.h"

TME_BEGIN_DECLS

/// Models strings.
typedef tme_struct(TmeString) {

    /// String buffer.
    char const* cstring;

    /// String length (excluding the null terminator).
    size_t length;

} TmeString;

/**
 * Creates a new string initialized with the specified string buffer.
 *
 * @param cstring String buffer.
 * @return String instance.
 *
 * @note The string buffer is not copied: it will be deallocated once
 *       tme_string_deinit() is called.
 *
 * @public @memberof TmeString
 */
TME_PUBLIC
TmeString tme_string_init(char const *cstring);

/**
 * Destroys the specified string, releasing its memory.
 *
 * @param string String to destroy.
 *
 * @public @memberof TmeString
 */
TME_PUBLIC
void tme_string_deinit(TmeString string);

/**
 * Checks whether two strings are equal.
 *
 * @param a LHS of the equality relation.
 * @param b RHS of the equality relation.
 * @return True if the strings are equal, false otherwise.
 *
 * @public @memberof TmeString
 */
TME_PUBLIC
bool tme_string_equals(TmeString a, TmeString b);

/**
 * Checks whether 'a' comes before 'b' in lexicographical order.
 *
 * @param a LHS of the comparison.
 * @param b RHS of the comparison.
 * @return True if 'a' comes before 'b', false otherwise.
 *
 * @public @memberof TmeString
 */
TME_PUBLIC
bool tme_string_compare(TmeString a, TmeString b);

/**
 * Compares 'a' and 'b' in lexicographical order.
 *
 * @param a LHS of the comparison.
 * @param b RHS of the comparison.
 * @return -1 if 'a' comes before 'b', 0 if 'a' is equal to 'b', 1 if 'a' comes after 'b'.
 *
 * @public @memberof TmeString
 */
TME_PUBLIC
int tme_string_cmp(TmeString a, TmeString b);

/**
 * Copies the specified string.
 *
 * @param string String to copy.
 * @return Copied string.
 *
 * @public @memberof TmeString
 */
TME_PUBLIC
TmeString tme_string_copy(TmeString string);

/**
 * Creates a new string by copying the specified string buffer.
 *
 * @param cstring String buffer.
 * @return String instance.
 *
 * @public @memberof TmeString
 */
TME_PUBLIC
TmeString tme_string_dup(char const *cstring);

/**
 * Creates a new string by repeating the specified string buffer multiple times.
 *
 * @param cstring String buffer.
 * @param times Repetitions.
 * @return String instance.
 *
 * @public @memberof TmeString
 */
TME_PUBLIC
TmeString tme_string_repeating(char const *cstring, tme_uint_t times);

/**
 * Creates a new string with the specified format.
 *
 * @param format Format string.
 * @param ... Format arguments.
 * @return String instance.
 *
 * @public @memberof TmeString
 */
TME_PUBLIC
TmeString tme_string_with_format(char const *format, ...);

TME_END_DECLS

#endif // TME_STRING_H
